package goaml.lang;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import goaml.lang.driver.Driver;
import goaml.lang.entity.Entity;
import goaml.lang.state.AbstractState;
import goaml.lang.util.CSVWriter;

public final class Simulation {

	private final StateChart stateChart;
	private final Driver driver;

	private final int TIME_MAX; // Currently, in hour
	
	public Simulation(final int timeMax, final String xmlPath, final String driverPath) {
		this.TIME_MAX = timeMax;

		stateChart = new StateChart(xmlPath);
		driver = new Driver(driverPath, stateChart);
	}

	public final void start() {

		// **************************************************
		// Update simulation and export data as CSV file
		// **************************************************
		CSVWriter csv = null;
		try {
			// Create CSV file
			csv = new CSVWriter(CSVWriter.SIMULATION, stateChart.getName());

			List<String> datas = new ArrayList<String>();

			// Gets states
			List<Entity> entities = stateChart.getEntities();
			for (Entity entity : entities) {
				List<AbstractState> states = entity.getStates();

				// For each state, generate a name
				for (AbstractState state : states) {
					datas.add(entity.getName() + "-" + entity.getId() + "-" + state.getName() + state.getNumCohort());
				}
			}
			// Insert name of the lowest states in the csv file
			csv.write(datas);

			datas = new ArrayList<String>();
			for (Entity entity : entities) {
				List<AbstractState> states = entity.getStates();

				// For each states, get their values
				for (AbstractState state : states) {
					datas.add(String.valueOf(state.getValue()));
				}
			}
			// Insert initials values of the states
			csv.write(datas);

			// Update the simulation and insert new values for each steps
			for (double time = 0; time < TIME_MAX; time += 1) {

				stateChart.update();
				driver.update(time);

				datas = new ArrayList<String>();
				for (Entity entity : entities) {
					List<AbstractState> states = entity.getStates();

					// For each states, get their values
					for (AbstractState state : states) {
						datas.add(String.valueOf(state.getValue()));
					}
				}
				// Insert initials values of the states
				csv.write(datas);
			}

			// Flush and close the CSV file
			csv.flush();
			csv.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		}

	}

}
