package goaml.lang.parameter;

import java.util.List;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import goaml.lang.state.State;

public final class Constant {

	public static final int DEFAULT_NUM_COHORT = -1;

	private final String NAME;
	private final double VALUE;
	private final String SOURCE_GUARD;
	private final int SOURCE_NUM_COHORT;
	private final String TARGET_GUARD;
	private final int TARGET_NUM_COHORT;

	private Constant(final String name, final double value, final String sourceGuard, final int sourceNumCohort,
			final String targetGuard, final int targetNumCohort) {
		this.NAME = name;
		this.VALUE = value;
		this.SOURCE_GUARD = sourceGuard;
		this.SOURCE_NUM_COHORT = sourceNumCohort;
		this.TARGET_GUARD = targetGuard;
		this.TARGET_NUM_COHORT = targetNumCohort;
	}

	// From ONE "Constant" tag, create and return an instance of Constant
	public static final Constant parse(final Node node) {
		if (node == null) {
			throw new RuntimeException("Node is null");
		}
		if (node.getNodeName() != "Constant") {
			throw new RuntimeException("NodeName \"Constant\" was expected, got : " + node.getNodeName());
		}

		String name = null;
		double value = 0.0;
		String sourceGuard = null;
		int sourceNumCohort = DEFAULT_NUM_COHORT; // -1 by default if sourceNumCohort is not present
		String targetGuard = null;
		int targetNumCohort = DEFAULT_NUM_COHORT; // -1 by default if targetNumCohort is not present

		NamedNodeMap nodeMap = node.getAttributes();

		// Required attributes
		if (nodeMap.getNamedItem("name") != null) {
			name = nodeMap.getNamedItem("name").getTextContent();
		} else {
			throw new RuntimeException("\"name\" attribute missed");
		}

		if (nodeMap.getNamedItem("value") != null) {
			if (Double.valueOf(nodeMap.getNamedItem("value").getTextContent()) >= 0) {
				value = Double.valueOf(nodeMap.getNamedItem("value").getTextContent());
			} else {
				throw new RuntimeException("\"value\" is negative in " + node.getNodeName() + " tag");
			}
		} else {
			throw new RuntimeException("\"value\" attribute missed");
		}

		// Optional attributes
		for (int i = 0; i < node.getChildNodes().getLength(); i++) {
			Node subNode = node.getChildNodes().item(i);
			NamedNodeMap subNodeMap = subNode.getAttributes();

			switch (subNode.getNodeName()) {
			case "sourceGuard":
				if (subNodeMap.getNamedItem("state") != null) {
					sourceGuard = subNodeMap.getNamedItem("state").getTextContent();
				} else {
					throw new RuntimeException("\"state\" attribute missed");
				}

				if (subNodeMap.getNamedItem("numCohortGuard") != null) {
					if (Integer.valueOf(subNodeMap.getNamedItem("numCohortGuard").getTextContent()) >= 0) {
						sourceNumCohort = Integer.valueOf(subNodeMap.getNamedItem("numCohortGuard").getTextContent());
					} else {
						throw new RuntimeException(
								"\"numCohortGuard\" is negative in " + subNode.getNodeName() + " tag");
					}
				}
				break;

			case "targetGuard":
				if (subNodeMap.getNamedItem("state") != null) {
					targetGuard = subNodeMap.getNamedItem("state").getTextContent();
				} else {
					throw new RuntimeException("\"state\" attribute missed");
				}

				if (subNodeMap.getNamedItem("numCohortGuard") != null) {
					if (Integer.valueOf(subNodeMap.getNamedItem("numCohortGuard").getTextContent()) >= 0) {
						targetNumCohort = Integer.valueOf(subNodeMap.getNamedItem("numCohortGuard").getTextContent());
					} else {
						throw new RuntimeException(
								"\"numCohortGuard\" is negative in " + subNode.getNodeName() + " tag");
					}
				}
				break;

			default:
				break;
			}
		}

		return new Constant(name, value, sourceGuard, sourceNumCohort, targetGuard, targetNumCohort);
	}

	// From a list of constant, get the constant which has the name "constantName"
	// With sourceGuard = source if sourceGuard is present
	// With sourceNumCohort = source.getNumCohort() if sourceGuard is present
	// With targetGuard = target if targetGuard is present
	// With targetNumCohort = target.getNumCohort() if targetGuard is present
	// Return null if there is no corresponding constant
	public static Constant getConstant(final String constantName, final List<Constant> constants, final State source,
			final State target) {
		for (Constant constant : constants) {
			if (constant.getSourceGuard() == null || constant.getSourceGuard().equals(source.getName())
					&& (constant.getSourceNumCohort() == Constant.DEFAULT_NUM_COHORT
							|| constant.getSourceNumCohort() == source.getNumCohort())) {
				if (constant.getTargetGuard() == null || constant.getTargetGuard().equals(target.getName())
						&& (constant.getTargetNumCohort() == Constant.DEFAULT_NUM_COHORT
								|| constant.getTargetNumCohort() == target.getNumCohort())) {
					if (constant.getName().equals(constantName)) {
						return constant;
					}
				}
			}
		}
		return null;
	}

	public final String getName() {
		return this.NAME;
	}

	public final double getValue() {
		return VALUE;
	}

	public final String getSourceGuard() {
		return this.SOURCE_GUARD;
	}

	public final int getSourceNumCohort() {
		return this.SOURCE_NUM_COHORT;
	}

	public final String getTargetGuard() {
		return this.TARGET_GUARD;
	}

	public final int getTargetNumCohort() {
		return this.TARGET_NUM_COHORT;
	}

}
