package goaml.lang.state;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import goaml.lang.entity.Entity;

public final class CompositeState extends AbstractState {

	private final List<AbstractState> STATES;

	protected CompositeState(final Entity entity, final Node node, final boolean canReceiveInitialValue) {
		if (node == null) {
			throw new RuntimeException("Node is null");
		}
		if (!node.hasChildNodes()) {
			throw new RuntimeException("A CompositeState must have children !");
		}

		NamedNodeMap nodeMap = node.getAttributes();

		// Required attributes
		if (nodeMap.getNamedItem("name") != null) {
			name = nodeMap.getNamedItem("name").getTextContent();
		} else {
			throw new RuntimeException("\"name\" attribute missed");
		}

		STATES = new ArrayList<AbstractState>();

		// Create every subStates
		for (int i = 0; i < node.getChildNodes().getLength(); i++) {
			STATES.add(AbstractState.parse(entity, node.getChildNodes().item(i), canReceiveInitialValue));
		}
	}

	@Override
	public final double getValue() {
		double value = 0.0;

		for (AbstractState state : this.STATES) {
			value += state.getValue();
		}

		return value;
	}

	public final List<AbstractState> getStates() {
		return this.STATES;
	}

}
