package goaml.lang.transition;

import java.util.List;

import goaml.lang.StateChart;
import goaml.lang.parameter.Constant;
import goaml.lang.parameter.Reference;
import goaml.lang.state.State;
import goaml.lang.util.DelayedUpdate;

public final class DivisionWithDifferentiation extends AbstractTransition {

	private Constant rate;
	private Constant probability;

	public DivisionWithDifferentiation() {
	}

	private DivisionWithDifferentiation(final State source, final State target, final Constant rate,
			final Constant probability) {
		super(source, target);
		this.rate = rate;
		this.probability = probability;
	}

	@Override
	protected void initialize(StateChart stateChart, State source, State target, List<Constant> constants,
			List<Reference> references) {

		// Initialize constants
		rate = Constant.getConstant("rate", constants, source, target);
		probability = Constant.getConstant("probability", constants, source, target);

		// Check if all attributes have been initialized
		if (rate == null || probability == null) {
			return;
		}

		stateChart.addTransition(new DivisionWithDifferentiation(source, target, rate, probability));

	}

	@Override
	public void compute(StateChart stateChart) {
		// Compute the rate
		double rate = this.rate.getValue() * this.probability.getValue() * this.source.getValue();

		// Create DelayedUpdates in order to apply computed value on the states when all transitions
		// and processes have compute
		stateChart.addDelayedUpdate(new DelayedUpdate(this.source, DelayedUpdate.Action.SUBSTRACT, rate));
		stateChart.addDelayedUpdate(new DelayedUpdate(this.target, DelayedUpdate.Action.ADD, 2 * rate));
	}

}
