package goaml.lang.transition;

import java.util.List;

import goaml.lang.StateChart;
import goaml.lang.parameter.Constant;
import goaml.lang.parameter.Reference;
import goaml.lang.state.State;
import goaml.lang.util.DelayedUpdate;

public final class Infection extends AbstractTransition {

	private Constant infectionRate;
	private Reference virus;

	private Infection(final State source, final State target, final Constant infectionRate, final Reference virus) {
		super(source, target);
		this.infectionRate = infectionRate;
		this.virus = virus;
	}

	@Override
	protected void initialize(StateChart stateChart, State source, State target, List<Constant> constants,
			List<Reference> references) {

		// Initialize constants
		infectionRate = Constant.getConstant("infectionRate", constants, source, target);

		for (Reference reference : references) {
			if (reference.getName().equals("virus")) {
				virus = reference;
			}

			// Check if all attributes have been initialized
			if (infectionRate == null || virus == null) {
				return;
			}

			stateChart.addTransition(new Infection(source, target, infectionRate, virus));
		}
	}

	@Override
	public void compute(StateChart stateChart) {
		// Compute the rate
		double rate = this.source.getValue() * this.infectionRate.getValue() * this.virus.getState().getValue();

		// Create DelayedUpdates in order to apply computed value on the states when all transitions
		// and processes have compute
		stateChart.addDelayedUpdate(new DelayedUpdate(this.source, DelayedUpdate.Action.SUBSTRACT, 0));
		stateChart.addDelayedUpdate(new DelayedUpdate(this.target, DelayedUpdate.Action.ADD, rate));
	}

}
