package goaml.lang.transition;

import java.util.ArrayList;
import java.util.List;

import goaml.lang.StateChart;
import goaml.lang.entity.Entity;
import goaml.lang.parameter.Constant;
import goaml.lang.parameter.Reference;
import goaml.lang.state.AbstractState;
import goaml.lang.state.State;
import goaml.lang.util.DelayedUpdate;

public final class SelectionWithProbability extends AbstractTransition {

	private Constant rate;
	private Constant probability;
	private List<AbstractState> population;

	public SelectionWithProbability() {
	}

	private SelectionWithProbability(final State source, final State target, final Constant rate, final Constant probability,
			final List<AbstractState> population) {
		super(source, target);
		this.rate = rate;
		this.probability = probability;
		this.population = population;
	}

	@Override
	protected void initialize(StateChart stateChart, State source, State target, List<Constant> constants,
			List<Reference> references) {

		// Initialize constants
		rate = Constant.getConstant("rate", constants, source, target);
		probability = Constant.getConstant("probability", constants, source, target);

		// Check if all attributes have been initialized
		if (rate == null && probability == null) {
			return;
		}

		population = new ArrayList<AbstractState>();

		List<Entity> entities = stateChart.getEntities();
		for (Entity entity : entities) {
			population.addAll(entity.getStatesByName(source.getName()));
		}

		stateChart.addTransition(
				new SelectionWithProbability(source, (State) source.getFollowingCohort(), rate, probability, population));
	}

	@Override
	public void compute(StateChart stateChart) {
		// Compute the rate
		double value = 0.0;
		for (AbstractState state : this.population) {
			value += state.getValue();
		}
		double rate = this.rate.getValue() * this.probability.getValue()* this.source.getValue();

		// Create DelayedUpdates in order to apply computed value on the states when all transitions
		// and processes have compute
		stateChart.addDelayedUpdate(new DelayedUpdate(this.source, DelayedUpdate.Action.SUBSTRACT, rate));
		stateChart.addDelayedUpdate(new DelayedUpdate(this.target, DelayedUpdate.Action.ADD, rate));
	}

}
