package goaml.lang.transition;

import java.util.ArrayList;
import java.util.List;

import goaml.lang.StateChart;
import goaml.lang.entity.Entity;
import goaml.lang.parameter.Constant;
import goaml.lang.parameter.Reference;
import goaml.lang.state.AbstractState;
import goaml.lang.state.State;
import goaml.lang.util.DelayedUpdate;

public final class TCRProlif extends AbstractTransition {

	private Constant proliferationRate;
	private Constant k;
	private List<AbstractState> population;

	public TCRProlif() {
	}

	private TCRProlif(final State source, final State target, final Constant proliferationRate, final Constant k,
			final List<AbstractState> population) {
		super(source, target);
		this.proliferationRate = proliferationRate;
		this.k = k;
		this.population = population;
	}

	@Override
	protected void initialize(StateChart stateChart, State source, State target, List<Constant> constants,
			List<Reference> references) {

		// Initialize constants
		proliferationRate = Constant.getConstant("proliferationRate", constants, source, target);
		k = Constant.getConstant("k", constants, source, target);

		// Check if all attributes have been initialized
		if (proliferationRate == null && k == null) {
			return;
		}

		population = new ArrayList<AbstractState>();

		List<Entity> entities = stateChart.getEntities();
		for (Entity entity : entities) {
			population.addAll(entity.getStatesByName(source.getName()));
		}

		stateChart.addTransition(
				new TCRProlif(source, (State) source.getFollowingCohort(), proliferationRate, k, population));
	}

	@Override
	public void compute(StateChart stateChart) {
		// Compute the rate
		double value = 0.0;
		for (AbstractState state : this.population) {
			value += state.getValue();
		}
		double rate = this.proliferationRate.getValue() * this.source.getValue() * (1 - value / this.k.getValue());

		// Create DelayedUpdates in order to apply computed value on the states when all transitions
		// and processes have compute
		stateChart.addDelayedUpdate(new DelayedUpdate(this.source, DelayedUpdate.Action.SUBSTRACT, rate));
		stateChart.addDelayedUpdate(new DelayedUpdate(this.target, DelayedUpdate.Action.ADD, 2 * rate));
	}

}
