package goaml.lang.transition;

import java.util.List;

import goaml.lang.StateChart;
import goaml.lang.parameter.Constant;
import goaml.lang.parameter.Reference;
import goaml.lang.state.State;
import goaml.lang.util.DelayedUpdate;

public final class VirusReplication extends AbstractTransition {

	private Constant replicationRate;
	private Reference infectedCell;

	private VirusReplication(final State source, final State target, final Constant replicationRate,
			final Reference infectedCell) {
		super(source, target);
		this.replicationRate = replicationRate;
		this.infectedCell = infectedCell;
	}

	@Override
	protected void initialize(StateChart stateChart, State source, State target, List<Constant> constants,
			List<Reference> references) {

		// Initialize constants
		replicationRate = Constant.getConstant("replicationRate", constants, source, target);

		for (Reference reference : references) {
			if (reference.getName().equals("infectedCell")) {
				infectedCell = reference;
			}

			// Check if all attributes have been initialized
			if (replicationRate == null || infectedCell == null) {
				return;
			}

			stateChart.addTransition(new VirusReplication(source, target, replicationRate, infectedCell));
		}
	}

	@Override
	public void compute(StateChart stateChart) {
		// Compute the rate
		double rate = this.infectedCell.getState().getValue() * this.replicationRate.getValue();

		// Create DelayedUpdates in order to apply computed value on the states when all transitions
		// and processes have compute
		stateChart.addDelayedUpdate(new DelayedUpdate(this.source, DelayedUpdate.Action.SUBSTRACT, 0));
		stateChart.addDelayedUpdate(new DelayedUpdate(this.target, DelayedUpdate.Action.ADD, rate));
	}

}
