package goaml.mvc.controller;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;

import goaml.lang.Simulation;
import goaml.mvc.model.SimulationModel;
import goaml.mvc.view.AppView;
import goaml.mvc.view.DisplayView;

public class AppController {

	// Models
	private SimulationModel simulationModel;

	// Views
	private AppView appView;

	public AppController() {
		this.simulationModel = new SimulationModel();

		this.appView = new AppView();

		// **************************************************
		// App view
		// **************************************************
		UpdateXMLPath updateXMLPath = new UpdateXMLPath();
		this.simulationModel.addObserver(updateXMLPath);
		this.appView.browseXMLButton.addActionListener(updateXMLPath);

		// Duration
		UpdateDuration updateDuration = new UpdateDuration();
		this.simulationModel.addObserver(updateDuration);
		this.appView.durationTextField.addKeyListener(updateDuration);
		
		// IntegralStep
		UpdateIntegralStep updateIntegralStep = new UpdateIntegralStep();
		this.simulationModel.addObserver(updateIntegralStep);
		this.appView.integrationTextField.addKeyListener(updateIntegralStep);
		

		// Time Unit
		UpdateTimeUnit updateTimeUnit = new UpdateTimeUnit();
		this.simulationModel.addObserver(updateTimeUnit);
		this.appView.timeUnitComboBox.addItemListener(updateTimeUnit);

		// Driver file
		UpdateDriverPath updateDriverPath = new UpdateDriverPath();
		this.simulationModel.addObserver(updateDriverPath);
		this.appView.browseDriverButton.addActionListener(updateDriverPath);

		DeleteDriverPath deleteDriverPath = new DeleteDriverPath();
		this.simulationModel.addObserver(deleteDriverPath);
		this.appView.deleteDriverButton.addActionListener(deleteDriverPath);

		// Simulate
		ActionSimulate actionSimulate = new ActionSimulate();
		this.appView.simulate.addActionListener(actionSimulate);

		// CSV file
		UpdateCSVPath updateCSVPath = new UpdateCSVPath();
		this.simulationModel.addObserver(updateCSVPath);
		this.appView.browseCSVButton.addActionListener(updateCSVPath);

		// Display
		ActionDisplay actionDisplay = new ActionDisplay();
		this.appView.display.addActionListener(actionDisplay);

		// Initialize the value of all JComponents with the value of the model
		this.simulationModel.init();
	}

	private class UpdateXMLPath implements Observer, ActionListener {

		@Override
		public void actionPerformed(ActionEvent arg0) {
			JFileChooser fileChooser = new JFileChooser(new File(simulationModel.getXMLPath()));
			FileNameExtensionFilter filter = new FileNameExtensionFilter("XML", "xml");
			fileChooser.setFileFilter(filter);
			if (fileChooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
				simulationModel.setXMLPath(fileChooser.getSelectedFile().getAbsolutePath());
			}
		}

		@Override
		public void update(Observable arg0, Object arg1) {
			appView.pathXMLTextField.setText(simulationModel.getXMLPath());
		}

	}

	private class UpdateDriverPath implements Observer, ActionListener {

		@Override
		public void actionPerformed(ActionEvent arg0) {
			JFileChooser fileChooser = new JFileChooser(new File(simulationModel.getDriverPath()));
			FileNameExtensionFilter filter = new FileNameExtensionFilter("CSV", "csv");
			fileChooser.setFileFilter(filter);
			if (fileChooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
				simulationModel.setDriverPath(fileChooser.getSelectedFile().getAbsolutePath());
			}
		}

		@Override
		public void update(Observable arg0, Object arg1) {
			appView.pathDriverTextField.setText(simulationModel.getDriverPath());
		}

	}

	private class DeleteDriverPath implements Observer, ActionListener {

		@Override
		public void actionPerformed(ActionEvent arg0) {
			simulationModel.setDriverPath("");
		}

		@Override
		public void update(Observable arg0, Object arg1) {
			appView.pathDriverTextField.setText(simulationModel.getDriverPath());
		}

	}

	private class UpdateDuration implements Observer, KeyListener {

		@Override
		public void update(Observable o, Object arg) {
			appView.durationTextField.setText(String.valueOf(simulationModel.getNbrOfTimeSteps()));
		}

		@Override
		public void keyPressed(KeyEvent arg0) {
		}

		@Override
		public void keyReleased(KeyEvent arg0) {
			int duration = simulationModel.DEFAULT_DURATION;
			try {
				duration = Integer.valueOf(appView.durationTextField.getText());
			} catch (Exception e) {
				// If the textfield contains non-numeric value or is empty,
				// the value of the duration is 1
			}
			simulationModel.setNbrOfTimeSteps(duration);
		}

		@Override
		public void keyTyped(KeyEvent arg0) {
		}

	}
	
	private class UpdateIntegralStep implements Observer, KeyListener {
		
		@Override
		public void update(Observable o, Object arg) {
			appView.integrationTextField.setText(String.valueOf(simulationModel.getIntegralStep()));
		}

		@Override
		public void keyPressed(KeyEvent arg0) {
		}

		@Override
		public void keyReleased(KeyEvent arg0) {
			double integralStep = simulationModel.DEFAULT_INTEGRALSTEP;
			try {
				integralStep = Double.valueOf(appView.integrationTextField.getText());
			} catch (Exception e) {
				// If the textfield contains non-numeric value or is empty,
				// the value of the duration is 1
			}
			simulationModel.setIntegralStep(integralStep);
		}

		@Override
		public void keyTyped(KeyEvent arg0) {
		}

	}

	private class UpdateTimeUnit implements Observer, ItemListener {

		@Override
		public void itemStateChanged(ItemEvent e) {
			simulationModel.setTimeUnit(
					SimulationModel.TimeUnit.valueOf(appView.timeUnitComboBox.getSelectedItem().toString()));
		}

		@Override
		public void update(Observable o, Object arg) {
			appView.timeUnitComboBox.setEditable(true);
			appView.timeUnitComboBox.setSelectedItem(simulationModel.getTimeUnit());
			appView.timeUnitComboBox.setEditable(false);
		}

	}

	private class ActionSimulate implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			int duration = simulationModel.getNbrOfTimeSteps();
			Simulation simulation = new Simulation(duration, simulationModel.getXMLPath(),
					simulationModel.getDriverPath());
			simulation.start();
		}

	}

	private class UpdateCSVPath implements Observer, ActionListener {

		@Override
		public void actionPerformed(ActionEvent arg0) {
			JFileChooser fileChooser = new JFileChooser(new File(simulationModel.getCSVPath()));
			FileNameExtensionFilter filter = new FileNameExtensionFilter("CSV", "csv");
			fileChooser.setFileFilter(filter);
			if (fileChooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
				simulationModel.setCSVPath(fileChooser.getSelectedFile().getAbsolutePath());
			}
		}

		@Override
		public void update(Observable arg0, Object arg1) {
			appView.pathCSVTextField.setText(simulationModel.getCSVPath());
		}

	}

	private class ActionDisplay implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			new DisplayView(simulationModel.getCSVPath());
		}

	}

}
